Package: SpatialDecon
Title: Deconvolution of mixed cells from spatial and/or bulk gene
        expression data
Version: 1.22.0
Authors@R: c(person("Maddy", "Griswold", email = "mgriswold@nanostring.com", role = c("cre", "aut")),
             person("Patrick", "Danaher", email = "pdanaher@nanostring.com", role = c("aut")))
Description: Using spatial or bulk gene expression data, estimates abundance of mixed cell types
 within each observation. Based on "Advances in mixed cell deconvolution enable quantification of 
 cell types in spatial transcriptomic data", Danaher (2022). Designed for use with
 the NanoString GeoMx platform, but applicable to any gene expression data. 
Depends: R (>= 4.0.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: TRUE
RoxygenNote: 7.2.3
Imports: grDevices, stats, utils, graphics, SeuratObject, Biobase,
        GeomxTools, repmis, methods, Matrix, logNormReg (>= 0.4)
Suggests: testthat, knitr, rmarkdown, qpdf, Seurat
biocViews: ImmunoOncology, FeatureExtraction, GeneExpression,
        Transcriptomics, Spatial
VignetteBuilder: knitr
BugReports: https://github.com/Nanostring-Biostats/SpatialDecon/issues
git_url: https://git.bioconductor.org/packages/SpatialDecon
git_branch: RELEASE_3_23
git_last_commit: 9ab4ca1
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:53:53 UTC; biocbuild
Author: Maddy Griswold [cre, aut],
  Patrick Danaher [aut]
Maintainer: Maddy Griswold <mgriswold@nanostring.com>
