# Generated by roxygen2: do not edit by hand

export(TIL_barplot)
export(collapseCellTypes)
export(create_profile_matrix)
export(derive_GeoMx_background)
export(download_profile_matrix)
export(florets)
export(mergeTumorIntoX)
export(reverseDecon)
export(runErrorModel)
export(spatialdecon)
exportMethods(runCollapseCellTypes)
exportMethods(runMergeTumorIntoX)
exportMethods(runReverseDecon)
exportMethods(runspatialdecon)
import(graphics)
import(stats)
import(utils)
importClassesFrom(GeomxTools,NanoStringGeoMxSet)
importFrom(Biobase,assayData)
importFrom(SeuratObject,GetAssayData)
importFrom(grDevices,colors)
importFrom(graphics,barplot)
importFrom(graphics,frame)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(logNormReg,lognlm)
importFrom(methods,is)
importFrom(repmis,source_data)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,lm)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(utils,data)
importFrom(utils,read.csv)
